---------------------------------------------------------------------------------------
-- Title          : Wishbone slave core for IRQ controller registers
---------------------------------------------------------------------------------------
-- File           : ../rtl/irq_controller_regs.vhd
-- Author         : auto-generated by wbgen2 from irq_controller_regs.wb
-- Created        : Tue May  7 14:57:57 2013
-- Standard       : VHDL'87
---------------------------------------------------------------------------------------
-- THIS FILE WAS GENERATED BY wbgen2 FROM SOURCE FILE irq_controller_regs.wb
-- DO NOT HAND-EDIT UNLESS IT'S ABSOLUTELY NECESSARY!
---------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity irq_controller_regs is
  port (
    rst_n_i                                  : in     std_logic;
    clk_sys_i                                : in     std_logic;
    wb_adr_i                                 : in     std_logic_vector(1 downto 0);
    wb_dat_i                                 : in     std_logic_vector(31 downto 0);
    wb_dat_o                                 : out    std_logic_vector(31 downto 0);
    wb_cyc_i                                 : in     std_logic;
    wb_sel_i                                 : in     std_logic_vector(3 downto 0);
    wb_stb_i                                 : in     std_logic;
    wb_we_i                                  : in     std_logic;
    wb_ack_o                                 : out    std_logic;
    wb_stall_o                               : out    std_logic;
-- Port for std_logic_vector field: 'Multiple interrupt' in reg: 'Multiple interrupt register'
    irq_ctrl_multi_irq_o                     : out    std_logic_vector(31 downto 0);
    irq_ctrl_multi_irq_i                     : in     std_logic_vector(31 downto 0);
    irq_ctrl_multi_irq_load_o                : out    std_logic;
-- Port for std_logic_vector field: 'Interrupt sources' in reg: 'Interrupt sources register '
    irq_ctrl_src_o                           : out    std_logic_vector(31 downto 0);
    irq_ctrl_src_i                           : in     std_logic_vector(31 downto 0);
    irq_ctrl_src_load_o                      : out    std_logic;
-- Port for std_logic_vector field: 'Interrupt enable mask' in reg: 'Interrupt enable mask register'
    irq_ctrl_en_mask_o                       : out    std_logic_vector(31 downto 0)
  );
end irq_controller_regs;

architecture syn of irq_controller_regs is

signal irq_ctrl_en_mask_int                     : std_logic_vector(31 downto 0);
signal ack_sreg                                 : std_logic_vector(9 downto 0);
signal rddata_reg                               : std_logic_vector(31 downto 0);
signal wrdata_reg                               : std_logic_vector(31 downto 0);
signal bwsel_reg                                : std_logic_vector(3 downto 0);
signal rwaddr_reg                               : std_logic_vector(1 downto 0);
signal ack_in_progress                          : std_logic      ;
signal wr_int                                   : std_logic      ;
signal rd_int                                   : std_logic      ;
signal allones                                  : std_logic_vector(31 downto 0);
signal allzeros                                 : std_logic_vector(31 downto 0);

begin
-- Some internal signals assignments. For (foreseen) compatibility with other bus standards.
  wrdata_reg <= wb_dat_i;
  bwsel_reg <= wb_sel_i;
  rd_int <= wb_cyc_i and (wb_stb_i and (not wb_we_i));
  wr_int <= wb_cyc_i and (wb_stb_i and wb_we_i);
  allones <= (others => '1');
  allzeros <= (others => '0');
-- 
-- Main register bank access process.
  process (clk_sys_i, rst_n_i)
  begin
    if (rst_n_i = '0') then 
      ack_sreg <= "0000000000";
      ack_in_progress <= '0';
      rddata_reg <= "00000000000000000000000000000000";
      irq_ctrl_multi_irq_load_o <= '0';
      irq_ctrl_src_load_o <= '0';
      irq_ctrl_en_mask_int <= "00000000000000000000000000000000";
    elsif rising_edge(clk_sys_i) then
-- advance the ACK generator shift register
      ack_sreg(8 downto 0) <= ack_sreg(9 downto 1);
      ack_sreg(9) <= '0';
      if (ack_in_progress = '1') then
        if (ack_sreg(0) = '1') then
          irq_ctrl_multi_irq_load_o <= '0';
          irq_ctrl_src_load_o <= '0';
          ack_in_progress <= '0';
        else
          irq_ctrl_multi_irq_load_o <= '0';
          irq_ctrl_src_load_o <= '0';
        end if;
      else
        if ((wb_cyc_i = '1') and (wb_stb_i = '1')) then
          case rwaddr_reg(1 downto 0) is
          when "00" => 
            if (wb_we_i = '1') then
              irq_ctrl_multi_irq_load_o <= '1';
            end if;
            rddata_reg(31 downto 0) <= irq_ctrl_multi_irq_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "01" => 
            if (wb_we_i = '1') then
              irq_ctrl_src_load_o <= '1';
            end if;
            rddata_reg(31 downto 0) <= irq_ctrl_src_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "10" => 
            if (wb_we_i = '1') then
              irq_ctrl_en_mask_int <= wrdata_reg(31 downto 0);
            end if;
            rddata_reg(31 downto 0) <= irq_ctrl_en_mask_int;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when others =>
-- prevent the slave from hanging the bus on invalid address
            ack_in_progress <= '1';
            ack_sreg(0) <= '1';
          end case;
        end if;
      end if;
    end if;
  end process;
  
  
-- Drive the data output bus
  wb_dat_o <= rddata_reg;
-- Multiple interrupt
  irq_ctrl_multi_irq_o <= wrdata_reg(31 downto 0);
-- Interrupt sources
  irq_ctrl_src_o <= wrdata_reg(31 downto 0);
-- Interrupt enable mask
  irq_ctrl_en_mask_o <= irq_ctrl_en_mask_int;
  rwaddr_reg <= wb_adr_i;
  wb_stall_o <= (not ack_sreg(0)) and (wb_stb_i and wb_cyc_i);
-- ACK signal generation. Just pass the LSB of ACK counter.
  wb_ack_o <= ack_sreg(0);
end syn;
